﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;

namespace Tsi1.BusinessLayer.Services
{
    public class BedeliaService : IBedeliaService
    {
        private readonly HttpClient _httpClient;

        public BedeliaService(HttpClient httpClient)
        {
            _httpClient = httpClient;
        }

        public async Task<ServiceResult<bool>> IsValidUser(string identityCard)
        {
            var result = new ServiceResult<bool>();

            var response = await _httpClient.GetAsync($"api/Users/{identityCard}");
            if (response.StatusCode != HttpStatusCode.OK)
            {
                result.HasError = true;
                var errorMessage = await response.Content.ReadAsStringAsync();
                result.AddMessage(errorMessage);
            }

            return result;
        }

        public async Task<ServiceResult<bool>> CloseRecord(string courseName, List<UserGradeDto> userGrades)
        {
            var result = new ServiceResult<bool>();
            var closeRecord = new CloseRecordDto()
            {
                CourseName = courseName,
                UserGrades = userGrades
            };

            var jsonInString = JsonConvert.SerializeObject(closeRecord);
            var model = new StringContent(jsonInString, Encoding.UTF8, "application/json");

            var response = await _httpClient.PostAsync("api/Courses/closeRecord", model);
            if (response.StatusCode != HttpStatusCode.OK)
            {
                result.HasError = true;
                var errorMessage = await response.Content.ReadAsStringAsync();
                result.AddMessage(errorMessage);
            }

            return result;
        }
    }
}
