﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class PostService : IPostService
    {
        private readonly Tsi1Context _context;

        private readonly IMapper _mapper;

        public PostService(Tsi1Context context, IMapper mapper)
        {
            _context = context;
            _mapper = mapper;
        }

        public async Task<ServiceResult<int>> Create(PostCreateDto newPost)
        {
            var result = new ServiceResult<int>();

            var post = _mapper.Map<Post>(newPost);

            post.Date = DateTime.Now;

            _context.Posts.Add(post);
            await _context.SaveChangesAsync();

            result.Data = post.Id;
            return result;
        }

        public async Task<ServiceResult<Post>> Delete(int postId)
        {
            var result = new ServiceResult<Post>();

            var post = await _context.Posts.FirstOrDefaultAsync(x => x.Id == postId);

            if (post == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.PostDoesNotExist, postId);
                return result;
            }

            _context.Posts.Remove(post);

            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<List<PostPreviewDto>>> GetPosts(int forumId)
        {
            var result = new ServiceResult<List<PostPreviewDto>>();

            var posts = await _context.Posts
                .Where(x => x.ForumId == forumId)
                .OrderByDescending(x => x.Date)
                .ToListAsync();

            var postDtos = _mapper.Map<List<PostPreviewDto>>(posts);

            var userIds = postDtos.Select(x => x.UserId).ToList();

            var users = await _context.Users.Where(x => userIds.Contains(x.Id)).ToListAsync();

            foreach (var postDto in postDtos)
            {
                var user = users.FirstOrDefault(x => x.Id == postDto.UserId);
                postDto.CompleteName = user.FirstName + " " + user.LastName;
            }

            result.Data = postDtos;

            return result;
        }
    }
}
